from enum import Enum

class RTDType(Enum):
    PT100 = 0
    PT500 = 1
    PT1000 = 2
    Cu10 = 3
    Cu50 = 4
    Cu100 = 4
    Ni100 = 6
    Ni120 = 7
    Ni1000 = 8

class TCType(Enum):
    E = 0
    J = 1
    K = 2
    N = 3
    R = 4
    S = 5
    T = 6
    B = 7

class ChannelType(Enum):
    @classmethod
    def RTD(rtd_type: RTDType): ...

    @classmethod
    def TC(tc_type: TCType): ...

class ChannelConfig:
    def __init__(
            self,
            channel_type: ChannelType,
            max_resistance: float,
            min_resistance: float,
            shunt_resistance: float,
            r_ain0: float
        ): ...


def set_channel_request(channel_number: int, channel_config: ChannelConfig, id: int) -> bytes: ...

def set_temperature_request(channel_number: int, temperature: float, id: int) -> bytes: ...



import serial

from rtd_common.channel.packets.set_channel import RTDType, ChannelType, ChannelConfig
from rtd_common import set_channel_request, set_temperature_request


def r_ain_0(channel: int):
    match channel:
        case 3:
            return 15.0
        case _:
            return 1.5e3


def set_channel(chan_number: int):
    channel_type = ChannelType.RTD(RTDType.PT100)
    channel_config = ChannelConfig(
        channel_type,
        390.0,
        0.0,
        12.0 + 0.033,
        r_ain_0(chan_number)
    )

    cmd = set_channel_request(chan_number, channel_config, 1)
    return cmd

def set_temperature(chan_number: int):
    j = 53000
    temperature = -225 + j / 65535 * 1700
    cmd = set_temperature_request(chan_number, temperature, j)
    return cmd

print(type(set_temperature(3)))

# with serial.Serial("/dev/ttyACM0",timeout=0.1) as rtd_board:
#     cmd = set_temperature(3)
#     print(cmd)
#     rtd_board.write(cmd)
#     response = rtd_board.readall()
#     print(response)


from hal_common.packets import Request as HalRequest
from rtd_common.channel.packets import Request as RTDRequest, GeneralRequest
from rtd_common.channel.packets.set_channels import (
    SetChannelRequest,
    ChannelConfig,
    ChannelType,
    RTDType
)


def r_ain_0(channel: int):
    match channel:
        case 3:
            return 15.0
        case _:
            return 1.5e3


def set_channel(chan_number: int):
    channel_type = ChannelType.RTD(RTDType.PT100)
    channel_config = ChannelConfig(
        channel_type,
        390.0,
        0.0,
        12.0 + 0.033,
        r_ain_0(chan_number)
    )

    set_channel_request = SetChannelRequest(channel_config)
    general_request = GeneralRequest.SetChannel(set_channel_request)
    rtd_request = RTDRequest.General(general_request)

    u8_c_number = chan_number & 0xFF

    msg = HalRequest.with_id(rtd_request, 1).into(u8_c_number).as_bytes()
    return msg
    # base_id: int = Request.General(general_request).base_id()
    # adress: int = chan_number
    # cmd: int = Request.General(general_request).cmd()
